@@ -6,7 +6,7 @@ Mojolicious::Plugin::CGI - Run CGI script from Mojolicious
 
 =head1 VERSION
 
-0.0501
+0.06
 
 =head1 DESCRIPTION
 
@@ -44,7 +44,7 @@ use constant CHUNK_SIZE => 131072;
 use constant CHECK_CHILD_INTERVAL => $ENV{CHECK_CHILD_INTERVAL} || 0.01;
 use constant DEBUG => $ENV{MOJO_PLUGIN_CGI_DEBUG} || 0;
 
-our $VERSION = '0.0501';
+our $VERSION = '0.06';
 our %ORIGINAL_ENV = %ENV;
 
 =head1 METHODS
@@ -93,9 +93,12 @@ sub emulate_environment {
 
   $script_name =~ s!^/?\Q$base_path\E/?!!;
 
+  my $content_length = $req->content->is_multipart
+    ? $req->body_size : $headers->content_length;
+
   return(
     %{ $self->env },
-    CONTENT_LENGTH => $headers->content_length || 0,
+    CONTENT_LENGTH => $content_length || 0,
     CONTENT_TYPE => $headers->content_type || '',
     GATEWAY_INTERFACE => 'CGI/1.1',
     HTTP_COOKIE => $headers->cookie || '',
@@ -158,7 +161,7 @@ sub register {
     my $c = shift->render_later;
     my $ioloop = Mojo::IOLoop->singleton;
     my $reactor = $ioloop->reactor;
-    my $stdin = $c->req->content->asset;
+    my $stdin;
     my $delay = $ioloop->delay;
     my($pid, $tid, $reader, $stdout_read, $stdout_write);
 
@@ -167,9 +170,13 @@ sub register {
     unless(pipe $stdout_read, $stdout_write) {
       return $c->render_exception("pipe: $!");
     }
-    unless($c->req->content->isa('Mojo::Content::Single')) {
-      return $c->render_exception('Can only handle Mojo::Content::Single requests');
+    if ($c->req->content->is_multipart) {
+      $stdin = Mojo::Asset::File->new;
+      $stdin->add_chunk($c->req->build_body);
+    } else {
+      $stdin = $c->req->content->asset;
     }
+
     unless($stdin->isa('Mojo::Asset::File')) {
       warn "Converting $stdin to Mojo::Asset::File\n" if DEBUG;
       $stdin = Mojo::Asset::File->new->add_chunk($stdin->slurp);
@@ -250,7 +257,3 @@ Jan Henning Thorsen - C<jhthorsen@cpan.org>
 =cut
 
 1;
-
-1;
-
-1;
